/* filepicker.p
 *
 * select a file
 *
 */

define input parameter        dirName  as character no-undo.
define input parameter        filter   as character no-undo.
define input-output parameter fileName as character no-undo format "x(40)".

define variable fileList as character no-undo label "Files:"
  view-as selection-list single sort size 40 by 12 scrollbar-vertical
.

define button ok     label "Ok"     auto-go.
define button cancel label "Cancel" auto-endkey.

define variable c as integer no-undo.

if current-window:width-chars < 160 then
  c = 20.
 else
  c = 60.

form
  skip(1)
  "  File:" skip
  fileName at 3 "  " skip
  skip(1)
  "  Files:" skip
  fileList at 3 "  " skip
  skip(1)
  ok at 10 " " cancel skip
  skip(1)
 with
  frame filePicker
  title " Choose File "
  no-labels
  overlay
  column c
  row (( screen-lines / 2 ) - 12 )
.

define stream dirStrm.


/* ui triggers
 */

on return, enter of fileName in frame filePicker do:
  apply "go" to frame filePicker.
  return no-apply.
end.

on entry, value-changed of fileName in frame filePicker do:
  define variable fList as character no-undo.
  run getFileList ( dirName, self:screen-value, input-output fList ).
  fileList:list-items = fList.
  fileList:visible = yes.
end.

on entry of fileList in frame filePicker do:
  fileName:screen-value = fileList:entry(1) no-error.
  return.
end.

on return, enter, default-action of fileList in frame filePicker do:
  if self:screen-value = "" or self:screen-value = ? then
    return no-apply.
   else
    fileName:screen-value = self:screen-value.
  /* apply "entry" to ok in frame filePicker. */	/* require an "ok"	*/
  apply "go" to frame filePicker.			/* GO right away	*/
end.

on value-changed of fileList in frame filePicker do:
  if self:screen-value = "" or self:screen-value = ? then
    return no-apply.
   else
    fileName:screen-value = self:screen-value.
end.


/* main block
 */

if dirName = "" or dirName = ? then
  dirName = ".".

file-info:file-name = dirName.
if ( file-info:full-pathname = ? ) or ( index( file-info:file-type, "d" ) = 0 ) then return.

if filter = "" or filter = ? then filter = "*".
fileName = filter.

do on stop undo, leave
   on error undo, leave
   on endkey undo, leave:

  update
    fileName fileList ok cancel
   with
   frame filePicker
  .

  hide frame filePicker no-pause.

  fileName = dirName + "/" + trim( fileName ).
  file-info:file-name = fileName.
  if ( file-info:full-pathname = ? ) then
    fileName = "".
   else
    fileName = file-info:full-pathname.
  return.

end.

hide frame filePicker no-pause.

fileName = "".

return.


procedure getFileList:

  define input        parameter dirName  as character no-undo.
  define input        parameter filter   as character no-undo.
  define input-output parameter fileList as character no-undo.

  define variable inFile   as character no-undo.
  define variable fileAttr as character no-undo.

  if dirName = "" or dirName = ? then
    dirName = ".".

  file-info:file-name = dirName.
  if ( file-info:full-pathname = ? ) or ( index( file-info:file-type, "d" ) = 0 ) then return.

  if filter = "" or filter = ? then filter = "*".

  fileList = "".

  input stream dirStrm from os-dir( dirName ).
  repeat:

    inFile = "".
    import stream dirStrm inFile ^ fileAttr.

    if index( fileAttr, "d" ) > 0 then next.

    if ( inFile matches filter ) then
      fileList = fileList + "," + inFile.

  end.
  input stream dirStrm close.

  fileList = trim( fileList, "," ).

  return.

end.
