/* lib/insproxy.p
 *
 */

define input  parameter envHost   as character no-undo.
define output parameter proxyStat as logical   no-undo.

define new global shared variable portalPort    as character no-undo.
define new global shared variable portalOptions as character no-undo.
define new global shared variable portalProxy   as character no-undo format "x(250)" view-as fill-in size 40 by 1 label "       Proxy".

define variable useSockets as logical no-undo.

if os-getenv( "USESOCKETS" ) = "no" then
  useSockets = no.
 else
  useSockets = yes.

/* initialize portal socket configuration
 */

portalPort = os-getenv( "PORTALPORT" ).
if portalPort = ? then portalPort = "".

if portalPort = "" then portalPort = "80".

portalOptions = os-getenv( "PORTALOPTIONS" ).
if portalOptions = ? then portalOptions = "".

/* set in lib/install.p
 *
 * portalProxy = os-getenv( "PROXY" ).
 * if portalProxy = ? then portalProxy = "".
 * 
 */

/*** already ran these persistently in lib/install.p!!!
 *
 * run ssg/sausage02.p persistent.
 * run ssg/sausage06.p persistent.
 */

proxyStat = false.

run proxyTest.

return.


procedure proxyTest:

  define variable result   as longchar  no-undo.
  define variable payload  as longchar  no-undo.

  define variable respHeaders as character no-undo.
  define variable resphdrX    as character no-undo.
  define variable httpHeaders as character no-undo.

  define variable ptVerStr    as character no-undo initial "{lib/ptversion.i} commit {lib/commit.i}".
  define variable NL          as character no-undo.

  ptVerStr = replace( ptVerStr, "~n", " " ).
  ptVerStr = replace( ptVerStr, "  ", " " ).
  ptVerstr = trim( ptVerStr ).

  NL = chr( 13 ) + chr( 10 ).						/* official http spec is CR + LF			*/

  proxy_loop: do while useSockets = yes
    on error undo, leave
    on endkey undo, leave
    on stop undo, leave:

    message "Checking communications with portal:" envHost portalPort portalProxy portalOptions.
    publish "logMsg" ( 0, substitute( "proxyTest(), checking communications with portal: &1 &2 &3 &4", envHost, portalPort, portalProxy, portalOptions )).

    /* curl -X GET http://dashboard.wss.com/proxy.test
     */

    run getURL(
      envHOST,			/* "demo.wss.com"		*/
      portalPort,		/* 80				*/
      "/proxy.test",		/* /proxy.test			*/
      '',			/* optional username:password	*/
      portalOptions,		/* options			*/
      portalProxy,		/* proxy			*/
      '',			/* optional extra headers	*/
      '',			/* fileName			*/
      output result,
      output respHeaders
    ).

  if result begins "test passed" then
    do:
      if portalProxy <> "" then
        message "Success: " portalProxy view-as alert-box title " proxyTest passed ".
      proxyStat = true.
      publish "logMsg" ( 0, "proxy test passed, portal communications are available." ).
      leave proxy_loop.
    end.
   else
    do:

      publish "logMsg" ( 0, "proxy test failed, portal communications are NOT available." ).
      publish "logMsg" ( 0, substring( entry( 1, result, "~n" ), 1, 80 )).

      display
        skip(1)
        "      The install process was unable to contact the portal server at" string( envHost ) format "x(30)"      skip
        skip(1)
        "      To independently verify, try a command similar to: " skip
        "          " substitute( "curl -vv -X GET http://&1/proxy.test", envHost ) format "x(65)" skip
        skip(1)
        "      You may need to use a proxy server or have your firewall rules modified in order to communicate     " skip
        "      with the portal.                                                                                    " skip
        skip(1)
        "      ProTop can communicate with servers that use proxy configurations similar to:                       " skip
        "            proxy.xyzzy.com:8080  or  user:pwd@xyzzy.com:8080                                             " skip
        skip(1)
        "      Enter your proxy server configuration or F4/^E to continue without portal communications.           " skip
        skip(1)
        portalProxy skip
        skip(1)
       with
        frame updProxy
        title " Proxy Server Configuration "
        centered
        row 3
        width 110
        side-labels
      .

      update portalProxy with frame updProxy.

    end.

  end.

  return.

end.
