@echo off

::  syncio.bat

::  Tests the storage subsystem's performance in synchronous writes, which Progress uses 
::  for the AI and BI files. This has been found to be a useful proxy for overall system 
::  performance and fitness for running databases.
 
::  This script creates a sports database, sets the BI cluster size to 16 MB, and then grows
::  the BI file by two additional clusters, for a total of 6 clusters (96 MB of writes).
 
::  Minimum acceptable result for a medium-demand server is 10MB/sec -- or 9.6 seconds
::   4 or 5 seconds is better
::   3 or less is "meh, ok, you don't need to throw the storage subsystem into a dumpster quite yet"
::   1 or 2 is really where you want to be nowadays

call #ENVPT3#\bin\protopenv.bat

if NOT "%1" == "" (
  set SYNCIODIR=%1
)

cd /d %SYNCIODIR% 2>NUL

FOR /F "tokens=* USEBACKQ" %%F IN (`type 12version ^| gawk "{ print $3 }" ^| gawk -F. "{print $1}"`) do (SET PROVERSION=%%F)

::  prior to 11.3, OpenEdge does not have (nor does it need) the -zextendSyncIO parameter
set ZEXTENDSYNCIO=-zextendSyncIO

::  override this default in older releases (9.x - 11.2.x)
if "%PROVERSION%" == "9" (
  set ZEXTENDSYNCIO=
)

if "%PROVERSION%" == "10" (
  set ZEXTENDSYNCIO=
)

FOR /F "tokens=* USEBACKQ" %%F IN (`type 12version ^| gawk "{ print $3 }" `) do (SET PROMAXMIN=%%F)

::  retain only the first four characters of PROMAXMIN (e.g. trim "11.2.1" to "11.2")
set PROMAXMIN=%PROMAXMIN:~0,4%

if "%PROMAXMIN%" == "11.0" (
        set ZEXTENDSYNCIO=
)

if "%PROMAXMIN%" == "11.1" (
        set ZEXTENDSYNCIO=
)

if "%PROMAXMIN%" == "11.2" (
        set ZEXTENDSYNCIO=
)

::  verify that there is not an existing sports db
::  if it exists it may not have been properly cleaned up or there might be a conflicting usage - either way we want to be cautious

if exist sports.db (

::  bad idea :(
::  echo y | call prodel sports > NUL 2>&1

  echo sports already exists, please check configuration!
    
  exit /b 1

)

::  create the temporary database and set the BI cluster size

call prodb sports sports > NUL 2>&1
call proutil sports -C truncate bi -bi 16384 > NUL 2>&1


::  open the DB and grow the BI area, and time the operation

ptime proutil sports -C bigrow 2 %ZEXTENDSYNCIO% | tail -1 | gawk "{print $3}" > syncio.tmp
set /p SYNCIO=<syncio.tmp

%PROTOP%\ubin\date "+%%Y/%%m/%%d %%T " > myts.tmp
set /p MYTS=<myts.tmp

echo %MYTS% %SYNCIO% %SYNCIODIR% | tee -a %LOGDIR%\syncio.log

echo.
echo 9 seconds =  10MB/sec -- anything longer and your disk subsystem is junk
echo 5 seconds =  20MB/sec -- not horrible
echo 3 seconds =  30MB/sec -- good
echo 1 second  = 100MB/sec -- excellent!
echo.

::  delete the temporary database 

echo y | call prodel sports > NUL 2>&1
del /q sports.st 2>NUL

cd /d %PROTOP%
